package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.IndicatorDictionaryMapper;
import com.ims.eval.dao.IndicatorMapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.IndicatorDictionary;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.service.IIndicatorService;
import io.swagger.models.auth.In;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@Service
public class IndicatorServiceImpl extends ServiceImpl<IndicatorMapper, Indicator> implements IIndicatorService {

	@Resource
	private IndicatorDictionaryMapper indicatorDictionaryMapper;

	@Override
	public IPage<Indicator> list(Integer pageNum, Integer pageSize, String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company) {

		QueryWrapper<Indicator> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<Indicator> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(Indicator::getId, id);
		}

		if (StringUtils.isNotEmpty(indicatorName)) {
			qw.lambda().like(Indicator::getIndicatorName, indicatorName);
		}

		if (StringUtils.isNotEmpty(indicatorCode)) {
			qw.lambda().like(Indicator::getIndicatorCode, indicatorCode);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(Indicator::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(Indicator::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(dept)) {
			qw.lambda().eq(Indicator::getDept, dept);
		}

		if (StringUtils.isNotEmpty(company)) {
			qw.lambda().eq(Indicator::getCompany, company);
		}

		qw.lambda().orderByAsc(Indicator::getOrderNum);

		IPage<Indicator> list = baseMapper.selectPage(page, qw);

		list.getRecords().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});

		return list;
	}

	@Override
	public IPage<Indicator> list2(Integer pageNum, Integer pageSize, String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		IPage<Indicator> list = baseMapper.seleclistPage(page, id, indicatorName, indicatorCode, binSection, binStage, dept, company);
		return list;
	}

	@Override
	public List<Indicator> listAll(String id, String indicatorName, String indicatorCode, String binSection, String binStage, String dept, String company) {
		QueryWrapper<Indicator> qw = new QueryWrapper<>();


		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(Indicator::getId, id);
		}

		if (StringUtils.isNotEmpty(indicatorName)) {
			qw.lambda().like(Indicator::getIndicatorName, indicatorName);
		}

		if (StringUtils.isNotEmpty(indicatorCode)) {
			qw.lambda().like(Indicator::getIndicatorCode, indicatorCode);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(Indicator::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(Indicator::getBinStage, binStage);
		}

		if (StringUtils.isNotEmpty(dept)) {
			qw.lambda().eq(Indicator::getDept, dept);
		}

		if (StringUtils.isNotEmpty(company)) {
			qw.lambda().eq(Indicator::getCompany, company);
		}
		qw.lambda().orderByAsc(Indicator::getOrderNum);

		List<Indicator> list = baseMapper.selectList(qw);

		list.stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});

		return list;
	}

	@Override
	public Indicator selectByCode(String indicatorCode) {

		Indicator indicator = baseMapper.selectByCode(indicatorCode);
		return indicator;
	}

	@Override
	public List<IndicatorResDTO> listByIds(List<String> indicatorIds) {
		List<IndicatorResDTO> list = baseMapper.selectListByIds(indicatorIds);
		return list;
	}

	@Override
	public boolean saveOrUpdate(Indicator entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<Indicator> qw = new QueryWrapper<>();
			qw.lambda().eq(Indicator::getIndicatorCode, entity.getIndicatorCode());
			qw.lambda().eq(Indicator::getBinSection, entity.getBinSection());
			qw.lambda().eq(Indicator::getBinStage, entity.getBinStage());
			List<Indicator> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前指标code已存在");
			}

			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);

	}


	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {
		idList.stream().forEach(i -> {
			QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, i);
			List<IndicatorDictionary> list = indicatorDictionaryMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				Indicator indicator = baseMapper.selectById(i);
				throw new RuntimeException(indicator.getIndicatorName() + "指标存在关联属性");
			}
		});
		int code = baseMapper.deleteBatchIds(idList);
		if (code > 0) {
			return true;
		}
		return false;
	}
}
